'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.These = undefined;

var _maybe = require('folktale/maybe');

/*::
declare function concatA<A, B, +Aʹ>(x: λThese<A, B>): λThese<A | Aʹ, B>
declare function concatB<A, B, +Bʹ>(x: λThese<A, B>): λThese<A, B | Bʹ>
declare function changeA<-A, B, +Aʹ>(x: λThese<A, B>): λThese<Aʹ, B>
declare function changeB<A, -B, +Bʹ>(x: λThese<A, B>): λThese<A, Bʹ>
declare function addTypes<A, +B, +C>(x: A): A | B | C
*/

class TheseCore {
  map(f) {
    return this.bimap(x => x, f);
  }
  /*::
  bimap<Aʹ, Bʹ>(f: (x: A) => Aʹ, g: (y: B) => Bʹ): λThese<Aʹ, Bʹ> {
    declare var ret: λThese<Aʹ, Bʹ>
    return ret
  }
  */
}

class This extends TheseCore {
  constructor(x) {
    super();
    this.x /*:: ; const xx*/ = x;
  }
  /*:: +*/
  bimap(f, g) {
    return new This(f(this.x));
  }
  left() {
    return (0, _maybe.Just)(this.x);
  }
  right() {
    return (0, _maybe.Nothing)();
  }
  cata({ This: fn }) {
    return (/*:: addTypes( */fn(this.x)
    ); /*:: ) */
  }
}

class That extends TheseCore {
  constructor(y) {
    super();
    this.y /*:: ; const xx*/ = y;
  }
  /*:: +*/
  bimap(f, g) {
    return new That(g(this.y));
  }
  left() {
    return (0, _maybe.Nothing)();
  }
  right() {
    return (0, _maybe.Just)(this.y);
  }
  cata({ That: fn }) {
    return (/*:: addTypes( */fn(this.y)
    ); /*:: ) */
  }
}

class Both extends TheseCore {
  /*:: +*/constructor(x, y) {
    super();
    this.x /*:: ; const xx*/ = x;
    this.y /*:: ; const yy*/ = y;
  }
  /*:: +*/
  bimap(f, g) {
    return new Both(f(this.x), g(this.y));
  }
  left() {
    return (0, _maybe.Just)(this.x);
  }
  right() {
    return (0, _maybe.Just)(this.y);
  }
  cata({ Both: fn }) {
    return (/*:: addTypes( */fn(this.x, this.y)
    ); /*:: ) */
  }
}

var These = exports.These = {
  This(x) {
    return new This(x);
  },
  That(y) {
    return new That(y);
  },
  Both(x, y) {
    return new Both(x, y);
  },
  of(y) {
    return new That(y);
  },
  thisOrBoth(x, y) {
    return y.matchWith({
      Just({ value: b }) {
        return new Both(x, b);
      },
      Nothing() {
        return new This(x);
      }
    });
  },
  thatOrBoth(x, y) {
    return y.matchWith({
      Just({ value: a }) {
        return new Both(a, x);
      },
      Nothing() {
        return new That(x);
      }
    });
  }
};

exports.default = These;
//# sourceMappingURL=these.js.map